class UnknownSize(Exception):
    pass


class ImageSize(object):
    def __init__(self):
        self.formats = []

    def register(self, format):
        self.formats.append(format)

    def get_size(self, fobj):
        for format in self.formats:
            if format.match(fobj):
                return format.get_size(fobj)
        raise UnknownSize()



from __future__ import absolute_import

import argparse
import io

import sys

from . import get_size, UnknownSize


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('path')
    args = parser.parse_args()
    with io.open(args.path, 'rb') as fobj:
        try:
            width, height = get_size(fobj)
        except UnknownSize:
            print("Could not determine image dimensions")
            sys.exit(1)
        else:
            print("%sx%s" % (width, height))


if __name__ == '__main__':
    main()



from __future__ import absolute_import

import struct

from . import SignatureFormat


class GIFSize(SignatureFormat):
    signature = struct.pack('4B', 0x47, 0x49, 0x46, 0x38)

    @classmethod
    def get_size(cls, fobj):
        signature = fobj.read(6)
        if signature not in (b'GIF87a', b'GIF89a'):
            raise ValueError("Invalid GIF signature %r" % signature)
        return struct.unpack('<HH', fobj.read(4))



from __future__ import absolute_import

import io
import json
import os
import unittest

from . import get_size

TEST_DATA_DIR = os.path.join(
    os.path.abspath(os.path.dirname(__file__)),
    'testdata',
)


class Base(unittest.TestCase):
    def assertImage(self, path):
        with open(path + '.json') as fobj:
            info = json.load(fobj)
        with io.open(path, 'rb') as fobj:
            width, height = get_size(fobj)
            got = {
                'width': width,
                'height': height
            }
            self.assertDictEqual(info, got)


def method_factory(path):
    def test_method(self):
        self.assertImage(path)
    test_method.__name__ = 'test_' + path
    test_method.__doc__ = path
    return test_method


def build():
    methods = {}
    for name in os.listdir(TEST_DATA_DIR):
        path = os.path.join(TEST_DATA_DIR, name)
        if os.path.exists(path + '.json'):
            methods['test_' + path] = method_factory(path)
    return type('Tests', (Base,), methods)


Tests = build()



from __future__ import absolute_import

import struct

from . import SignatureFormat


class BMPSize(SignatureFormat):
    signature = struct.pack('<2B', 0x42, 0x4D)

    @classmethod
    def get_size(cls, fobj):
        signature = fobj.read(2)
        if signature != cls.signature:
            raise ValueError("Invalid BMP signature %r" % signature)
        fobj.read(4)  # length
        fobj.read(4)  # reserved
        fobj.read(4)  # offset
        fobj.read(4)  # header size
        (width, height, nplanes, bits_per_pixel, compression_method, bmp_bytesz,
         hres, vres, ncolors, nimpcolors
         ) = struct.unpack_from('<IihhiiIIii', fobj.read(36))
        if nplanes != 1:
            raise ValueError("Unexpected nplanes %s" % nplanes)
        return width, height



from __future__ import absolute_import

from ..core import UnknownSize


class Format(object):
    @classmethod
    def match(cls, fobj):
        return False

    @classmethod
    def get_size(cls, fobj):
        raise UnknownSize()


class SignatureFormat(Format):
    signature = None
    @classmethod
    def match(cls, fobj):
        length = len(cls.signature)
        data = fobj.peek(length)[:length]
        return data == cls.signature



from setuptools import setup, find_packages

setup(
    name='imgsize',
    author='Jonas Obrist',
    license='BSD',
    version='1.1',
    test_suite='imgsize.tests',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ]
)




from __future__ import absolute_import

import struct

import zlib

from . import SignatureFormat
from ..core import UnknownSize


class PNGSize(SignatureFormat):
    signature = struct.pack(
        '8B', 0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a
    )

    max_chunk_length = 2 ** 31 - 1
    verify_constant = 2 ** 32 - 1
    chunk_ihdr = b'IHDR'

    @classmethod
    def get_size(cls, fobj):
        def sread(num):
            """
            reads num bytes and checks that the returned length matches
            """
            data = fobj.read(num)
            if len(data) != num:
                raise ValueError(
                    "Expected to read %s, got %s instead" % (num, len(data))
                )
            return data

        signature = fobj.read(8)
        if signature != cls.signature:
            raise ValueError("Invalid signature %r" % signature)

        width, height = None, None

        while width is None or height is None:
            raw_chunk_length = sread(4)
            chunk_length = struct.unpack('!I', raw_chunk_length)[0]
            if chunk_length > cls.max_chunk_length:
                raise ValueError("Chunk too long")
            raw_chunk_type = sread(4)
            bytes_chunk_type = struct.unpack('!4s', raw_chunk_type)[0]
            chunk_type = bytes_chunk_type
            data = sread(chunk_length)
            checksum = sread(4)
            verify = zlib.crc32(data, zlib.crc32(bytes_chunk_type))
            verify &= cls.verify_constant
            verify = struct.pack('!I', verify)
            if checksum != verify:
                raise ValueError("Checksum error")
            if chunk_type == cls.chunk_ihdr:  # IHDR = Image HeaDeR
                if chunk_length != 13:
                    raise ValueError("Invalid IHDR length")
                (width, height, bit_depth, color_type, compression_method,
                 filter_method, interlace_method) = struct.unpack('!2I5B', data)
                return width, height
        raise UnknownSize()



from __future__ import absolute_import

import struct

from . import SignatureFormat
from ..core import UnknownSize


class JPGSize(SignatureFormat):
    signature = struct.pack('3B', 0xff, 0xd8, 0xff)

    soi = 0xd8
    sof0 = 0xc0
    sof1 = 0xc1

    @classmethod
    def get_size(cls, fobj):
        def readbyte():
            return ord(fobj.read(1))

        def readword():
            return (readbyte() << 8) + readbyte()

        def readmarker():
            if readbyte() != 255:
                raise ValueError("Invalid marker")
            return readbyte()

        def skipmarker():
            length = readword()
            fobj.read(length - 2)

        width, height = None, None

        while width is None or height is None:
            marker = readmarker()
            if marker == cls.soi:  # Start Of Image
                pass
            elif marker in (cls.sof0, cls.sof1):
                readword()  # length
                readbyte()  # precision
                height = readword()
                width = readword()
                return width, height
            else:
                skipmarker()
        raise UnknownSize()



from __future__ import absolute_import

from .core import ImageSize, UnknownSize
from .formats import jpg, gif, png, bmp

__all__ = (
    'get_size',
    'UnknownSize',
)

DEFAULT = ImageSize()
DEFAULT.register(jpg.JPGSize)
DEFAULT.register(gif.GIFSize)
DEFAULT.register(png.PNGSize)
DEFAULT.register(bmp.BMPSize)

get_size = DEFAULT.get_size
